require 'spec_helper'

describe OO::Configuration do
  describe '.new' do
    let(:configuration) { described_class.new }
    let(:default_license_max_depth) { 4 }
    let(:default_license_keys) { [] }
    let(:default_license_missing_reaction) do
      -> { :license_missing_reaction_not_configured }
    end

    let(:default_license_checkers_default_user) do
      -> { :license_checkers_default_user_not_configured }
    end

    let(:default_license_checkers_default_key) do
      -> { :license_checkers_default_key_not_configured }
    end

    subject { configuration }

    it { is_expected.to be_a(described_class) }
    describe 'it is set to default values' do
      describe 'license_max_depth' do
        subject { configuration.license_max_depth }
        it { is_expected.to eq(default_license_max_depth) }
      end

      describe 'default_license_keys' do
        subject { configuration.default_license_keys }
        it { is_expected.to eq(default_license_keys) }
      end

      describe 'license_missing_reaction' do
        let(:license_missing_reaction) { configuration.license_missing_reaction }
        subject { license_missing_reaction }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { license_missing_reaction.call }
          it { is_expected.to eq(:license_missing_reaction_not_configured) }
        end
      end

      describe 'license_checkers_default_user' do
        let(:license_checkers_default_user) { configuration.license_checkers_default_user }
        subject { license_checkers_default_user }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { license_checkers_default_user.call }
          it { is_expected.to eq(:license_checkers_default_user_not_configured) }
        end
      end

      describe 'license_checkers_default_key' do
        let(:license_checkers_default_key) { configuration.license_checkers_default_key }
        subject { license_checkers_default_key }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { license_checkers_default_key.call }
          it { is_expected.to eq(:license_checkers_default_key_not_configured) }
        end
      end
    end

    describe '#license_max_depth=(depth)' do
      context 'when depth is not a fixnum' do
        let(:depth) { 'a' }
        let(:error) { OO::Errors::LicenseMaxDepthInvalid }

        subject { -> { configuration.license_max_depth = depth } }
        it { is_expected.to raise_error(error) }
      end

      context 'when depth is not greater than 0' do
        let(:depth) { 0 }
        let(:error) { OO::Errors::LicenseMaxDepthInvalid }

        subject { -> { configuration.license_max_depth = depth } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#license_missing_reaction=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenseMissingReactionInvalid }

        subject { -> { configuration.license_missing_reaction = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#license_checkers_default_user=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenseCheckersDefaultUserInvalid }

        subject { -> { configuration.license_checkers_default_user = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#license_checkers_default_key=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenseCheckersDefaultKeyInvalid }

        subject { -> { configuration.license_checkers_default_key = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#default_license_keys=(license_keys)' do
      context 'when license_keys is not an array' do
        let(:license_keys) { :invalid }
        let(:error) { OO::Errors::DefaultLicenseKeysInvalid }

        subject { -> { configuration.default_license_keys = license_keys } }
        it { is_expected.to raise_error(error) }
      end
    end
  end
end
